/**
 * @file
 * Dialog API inspired by HTML5 dialog element.
 *
 * @see http://www.whatwg.org/specs/web-apps/current-work/multipage/commands.html#the-dialog-element
 */

(function ($, Drupal, drupalSettings) {

  'use strict';

  /**
   * Default dialog options.
   *
   * @type {object}
   *
   * @prop {bool} [autoOpen=true]
   * @prop {string} [dialogClass='']
   * @prop {string} [buttonClass='button']
   * @prop {string} [buttonPrimaryClass='button--primary']
   * @prop {function} close
   */
  drupalSettings.dialog = {
    autoOpen: true,
    dialogClass: '',
    // Drupal-specific extensions: see dialog.jquery-ui.js.
    buttonClass: 'button',
    buttonPrimaryClass: 'button--primary',
    // When using this API directly (when generating dialogs on the client
    // side), you may want to override this method and do
    // `jQuery(event.target).remove()` as well, to remove the dialog on
    // closing.
    close: function (event) {
      Drupal.dialog(event.target).close();
      Drupal.detachBehaviors(event.target, null, 'unload');
    }
  };

  /**
   * @typedef {object} Drupal.dialog~dialogDefinition
   *
   * @prop {boolean} open
   *   Is the dialog open or not.
   * @prop {*} returnValue
   *   Return value of the dialog.
   * @prop {function} show
   *   Method to display the dialog on the page.
   * @prop {function} showModal
   *   Method to display the dialog as a modal on the page.
   * @prop {function} close
   *   Method to hide the dialog from the page.
   */

  /**
   * Polyfill HTML5 dialog element with jQueryUI.
   *
   * @param {HTMLElement} element
   *   The element that holds the dialog.
   * @param {object} options
   *   jQuery UI options to be passed to the dialog.
   *
   * @return {Drupal.dialog~dialogDefinition}
   *   The dialog instance.
   */
  Drupal.dialog = function (element, options) {
    var undef;
    var $element = $(element);
    var $body = $('body');
    var originalBodyPad = '';

    var dialog = {
      open: false,
      returnValue: undef,
      show: function () {
        openDialog({modal: false});
      },
      showModal: function () {
        openDialog({modal: true});
      },
      close: closeDialog
    };

    function openDialog(settings) {
      settings = $.extend({}, drupalSettings.dialog, options, settings);
      // Trigger a global event to allow scripts to bind events to the dialog.
      $(window).trigger('dialog:beforecreate', [dialog, $element, settings]);

      // Store the current body padding to undo any changes we may do.
      originalBodyPad = $body.css('padding-right');
      $body.data('drupal-dialog-padding', originalBodyPad);

      // Add body classes to indicate that the dialog is open.
      $body.addClass('drupal-dialog-open');
      // Add an additional class if a modal is open.
      if (settings.modal) {
        setScrollbarPadding();
        $body.addClass('drupal-dialog-open-modal');
      }

      $element.dialog(settings);
      dialog.open = true;
      $(window).trigger('dialog:aftercreate', [dialog, $element, settings]);
    }

    function closeDialog(value) {
      $(window).trigger('dialog:beforeclose', [dialog, $element]);

      // Remove the body classes we add in openDialog().
      $body.removeClass('drupal-dialog-open drupal-dialog-open-modal');

      // Restore original padding-right value for the body.
      originalBodyPad = $body.data('drupal-dialog-padding');
      $body.css('padding-right', originalBodyPad);

      $element.dialog('close');
      dialog.returnValue = value;
      dialog.open = false;
      $(window).trigger('dialog:afterclose', [dialog, $element]);
    }

    function isBodyOverflowing() {
      var fullWindowWidth = window.innerWidth;
      // Workaround for missing window.innerWidth in IE8.
      if (!fullWindowWidth) {
        var documentElementRect = document.documentElement.getBoundingClientRect();
        fullWindowWidth = documentElementRect.right - Math.abs(documentElementRect.left);
      }
      return document.body.clientWidth < fullWindowWidth;
    }

    function getScrollbarWidth() {
      // Create the measurement node.
      var scrollDiv = document.createElement('div');
      scrollDiv.className = 'drupal-dialog-scrollbar-measure';
      document.body.appendChild(scrollDiv);
      // Get the scrollbar width.
      var width = scrollDiv.offsetWidth - scrollDiv.clientWidth;
      // Delete the DIV.
      document.body.removeChild(scrollDiv);
      return width;
    }

    function setScrollbarPadding() {
      // Add padding to the body based on how wide our scrollbar is.
      var bodyPad = parseInt(($body.css('padding-right') || 0), 10);
      if (isBodyOverflowing()) $body.css('padding-right', bodyPad + getScrollbarWidth());
    }

    return dialog;
  };

})(jQuery, Drupal, drupalSettings);
